/**
 * This class defines the board in a "connect 4 game". It contains an array
 * of all of the pieces on the board, as well as methods that allow users 
 * of the class to add piece to the board, and check if one of the players
 * has won the game.
 * @author Kenneth Marino
 * @version 1.0 3/14/2012
 */
public class Board implements BoardInterface, WinCondition
{
	// Constants
	public static final int ROWS = 8;
	public static final int COLS = 8;
	
	// Instance variables
	private Piece[][] board;
	
	/**
	 * Constructor. Creates an array of pieces to represent the board. Initially,
	 * all pieces are null.
	 */
	public Board()
	{
		board = new Piece[ROWS][COLS];
	}
	
	/**
	 * This method allows users of Board to attempt to add a Piece to the board at
	 * a specified row. Does all of the checks necessary to make sure the move is 
	 * legal, and then makes it. Also empties the column if it becomes full.
	 * @param col The column to drop in a piece
	 * @param playerPiece The Piece to be added
	 * @return true if the move was sucessful, false otherwise
	 */
	public boolean move(int col, Piece playerPiece)
	{
		// Make sure column is valid
		if (col < 0 || col >= COLS)
			return false;
		// Make sure the Piece is not null
		if (playerPiece == null)
			return false;
		// Place piece in lowest occupied slot in column
		for (int row = ROWS-1; row >= 0; row--)
		{
			if (board[row][col] == null)
			{
				// Add the piece
				board[row][col] = playerPiece;
				// If the row is now full, we want to drop the column
				if (row == 0)
					dropColumn(col);
				return true;
			}
		}		
		// Otherwise return false
		return false;
	}

	/**
	 * This method clears the given row. It checks to ensure that there is not a
	 * win on the board, and then deletes all of the pieces from the given column.
	 * @param col The column to drop
	 * @return true if the column was dropped sucessfully 
	 */
	public boolean dropColumn(int col)
	{
		// Does not clear if somebody has just won
		if (checkForWin())
			return false;
		// Make sure column is valid
		if (col < 0 && col >= COLS)
			return false;
		// Clear out the given column
		for (int row = 0; row < ROWS; row++)
		{
			board[row][col] = null;
		}	
		return true;
	}
	
	/**
	 * This method checks the board for a winning condition. The four winning conditions are
	 * 4 pieces connected vertically, horizontally, diagonally left, and diagonally right.
	 * this method goes through each of the slots and checks for winnind conditions where
	 * a valid solution could possibly exist. 
	 * @return true if a solution has been found
	 */
	public boolean checkForWin()
	{
		// First check for any possible horizontal wins
		// Look down all rows
		for (int row = 0; row < ROWS; row++)
		{
			// Check all of the columns until within three of the column limit
			for (int col = 0; col < COLS - 3; col++)
			{
				// Return true if a win is found
				if (checkHorizontal(row, col))
					return true;
			}
		}
		
		// Next check for any possible vertical wins
		// Look across all columns
		for (int col = 0; col < COLS; col++)
		{
			// Check all of the rows until within three of the row limit
			for (int row = 0; row < ROWS - 3; row++)
			{
				// Return true if a win is found
				if (checkVertical(row, col))
				{
					return true;
				}
			}
		}
		
		// Next check for any right diagonal wins
		// Look across all columns until within three of the column limit
		for (int col = 0; col < COLS - 3; col++)
		{
			// Check all of the rows until within three of the row limit
 			for (int row = 0; row < ROWS - 3; row++)
			{
				if (checkDiagonalRight(row, col))
				{
					return true;
				}
			}
		}
		
		// Next check for any left diagonal wins
		// Look across all columns from 3 until the end
		for (int col = 3; col < COLS; col++)
		{
			// Check all of the rows until within three of the row limit
 			for (int row = 0; row < ROWS - 3; row++)
			{
				if (checkDiagonalLeft(row, col))
				{
					return true;
				}
			}
		}
		
		// If it was not found, return false
		return false;
	}
	
	/**
	 * This method return the state of the board, ie the 2D array of pieces representing
	 * the arrangement of the board. This will give the UI's the ability to show the board,
	 * but sends a copy so that the internal state cannot be altered.
	 * @return A 2D array of Pieces on the board
	 */
	public Piece[][] getState()
	{
		// Make a new array
		Piece[][] newBoard = new Piece[ROWS][COLS];
		// Copy the array to the new array
		for (int row = 0; row < ROWS; row++)
		{
			for (int col = 0; col < COLS; col++)
			{
				newBoard[row][col] = board[row][col];
			}
		}
		return newBoard;
	}

	/**
	 * This method allows you to reset the board to start the game over
	 * without having to redefine the Pieces.
	 */
	public void reset()
	{
		// Set all of the board to nulls
		for (int row = 0; row < ROWS; row++)
		{
			for (int col = 0; col < COLS; col++)
			{
				board[row][col] = null;
			}
		}
	}
	
	/**
	 * This method takes in a starting row and column position, and checks to see if there is a 
	 * horizontal win to the right of that initial position. Returns true if there is.
	 * @param row row position to check
	 * @param col starting column position to check
	 * @return true if all 4 pieces are owned by the same player, false otherwise
	 */
	public boolean checkHorizontal(int row, int col)
	{
		// Checks to see if the four pieces the the right starting at (row, col) are the same
		Piece comparePiece = board[row][col];
		// Return false if the first location doesn't have a piece
		if (comparePiece == null)
			return false;
		for (int i = 1; i < 4; i++)
		{
			// Return if the current piece is not the same as the first
			Piece curPiece = board[row][col+i];
			if (curPiece == null || !comparePiece.equals(curPiece))
				return false;
		}
		// Check all of them, they are all equal and not null
		return true;
	}
	
	/**
	 * This method takes in a starting row and column position, and checks to see if there is a 
	 * vertical win below the initial position. Returns true if there is.
	 * @param row starting row position to check
	 * @param col column position to check
	 * @return true if all 4 pieces are owned by the same player, false otherwise
	 */
	public boolean checkVertical(int row, int col)
	{
		// Checks to see if the four pieces below starting at (row, col) are the same
		Piece comparePiece = board[row][col];
		// Return false if the first location doesn't have a piece
		if (comparePiece == null)
			return false;
		for (int i = 1; i < 4; i++)
		{
			// Return if the current piece is not the same as the first
			Piece curPiece = board[row+i][col];
			if (curPiece == null || !comparePiece.equals(curPiece))
				return false;
		}
		// Check all of them, they are all equal and not null
		return true;
	}
	
	/**
	 * This method takes in a starting row and column position, and checks to see if there is a
	 * diagonal win down and to the right. Returns true if there is.	 
	 * @param row starting row position to check
	 * @param col starting column position to check
	 * @return true if all 4 pieces are owned by the same player, false otherwise
	 */
	public boolean checkDiagonalRight(int row, int col)
	{
		// Checks to see if the four pieces down and right starting at (row, col) are the same
		Piece comparePiece = board[row][col];
		// Return false if the first location doesn't have a piece
		if (comparePiece == null)
			return false;
		for (int i = 1; i < 4; i++)
		{
			// Return if the current piece is not the same as the first
			Piece curPiece = board[row+i][col+i];
			if (curPiece == null || !comparePiece.equals(curPiece))
				return false;
		}
		// Check all of them, they are all equal and not null
		return true;
	}
	
	/**
	 * This method takes in a starting row and column position, and checks to see if there is a
	 * diagonal win down and to the left. Returns true if there is.
	 * @param row starting row position to check
	 * @param col starting column position to check
	 * @return true if all 4 pieces are owned by the same player, false otherwise
	 */
	public boolean checkDiagonalLeft(int row, int col)
	{
		// Checks to see if the four pieces down and left starting at (row, col) are the same
		Piece comparePiece = board[row][col];
		// Return false if the first location doesn't have a piece
		if (comparePiece == null)
			return false;
		for (int i = 1; i < 4; i++)
		{
			// Return if the current piece is not the same as the first
			Piece curPiece = board[row+i][col-i];
			if (curPiece == null || !comparePiece.equals(curPiece))
				return false;
		}
		// Check all of them, they are all equal and not null
		return true;
	}
}