import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class defines the GUI in a "connect 4 game". It includes all of the 
 * members and functions for a UI which takes in user input on where to drop pieces
 * and displays the board in a meaningful way.
 * @author Kenneth Marino
 * @version 1.0 3/14/2012
 */
public class Panel4 extends JPanel
{
	// Instance variables - UI components
	private JLabel Player1Label;
	private JLabel Player2Label;
	private JLabel PlayerTurn;
	private JLabel MessageLabel;
	private JButton[] ColButtons;
	private JLabel[][] BoardIcons;
	private JPanel boardPanel;
	private ImageIcon blankImg;
	private ImageIcon OImg;
	private ImageIcon XImg;
	
	// Instance variables - state variables
	private Player player1;
	private Player player2;
	private Board board;
	private boolean p1Turn;
	
	/**
	 * Instantiates the panel. Creates all of the GUI components, and 
	 * initializes them to their beginning values. Also adds the action 
	 * listeners to the buttons, and sets the layout managers.
	 * @param p1Name The first player's name
	 * @param p2Name The second player's name
	 */
	public Panel4(String p1Name, String p2Name)
	{
		// Set to Box Layout
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	
		// Get image icons from files
		blankImg = new ImageIcon("blank.png");
		OImg = new ImageIcon("O.jpg");
		XImg = new ImageIcon("X.gif");
	
		// Set instance variables
		player1 = new Player(p1Name, 'X', XImg);
		player2 = new Player(p2Name, 'O', OImg);
		board = new Board();
		p1Turn = true;
		
		// Create Labels
		Player1Label = new JLabel("Player 1: " + p1Name);
		Player2Label = new JLabel("Player 2: " + p2Name);
		PlayerTurn = new JLabel(p1Name + "'s turn!");
		MessageLabel = new JLabel("Press a column button to drop a piece");
		
		// Create board panel
		GridLayout boardLayout = new GridLayout(Board.ROWS+1,Board.COLS);
		boardPanel = new JPanel(boardLayout);
		
		// Create BoardIcons
		BoardIcons = new JLabel[Board.ROWS][Board.COLS];
		for (int row = 0; row < Board.ROWS; row++)
		{
			for (int col = 0; col < Board.COLS; col++)
			{
				BoardIcons[row][col] = new JLabel(blankImg);
				boardPanel.add(BoardIcons[row][col]);
			}
		}
		
		// Create column buttons
		ColButtons = new JButton[Board.COLS];
		for (int i = 0; i < ColButtons.length; i++)
		{
			ColButtons[i] = new JButton(""+(i+1));
			ColButtons[i].addActionListener(new ColumnButtonListener(i));
			boardPanel.add(ColButtons[i]);
		}	
		
		// Add components to the panel
		add(Player1Label);
		add(Player2Label);
		add(PlayerTurn);	
		add(boardPanel);
		add(MessageLabel);	
	}
	
	/**
	 * This method updates the GUI to show an updated form of the board.
	 * Uses Board.getState to redraw the pieces on the GUI.
	 */
	private void updateBoard()
	{
		// Get the current board locations
		Piece[][] pieces = board.getState();
		// Goes through each Piece
		for (int row = 0; row < Board.ROWS; row++)
		{
			for (int col = 0; col < Board.COLS; col++)
			{
				// If it's null, put a blank there
				if (pieces[row][col] == null)
					BoardIcons[row][col].setIcon(blankImg);
				// If it's a real piece, use the getImageIcon method to display the image
				else 
					BoardIcons[row][col].setIcon(pieces[row][col].getImageIcon());				
			}
		}
	}
	
	/**
	 * This method disables the UI buttons so that the GUI
	 * can non longer be updated. Should be called after the user
	 * declines to play another game.
	 */
	private void disableButtons()
	{
		// Set all of the buttons to not be enabled
		for (JButton button : ColButtons)
		{
			button.setEnabled(false);
		}
		// Let the user know that they choose to exit
		MessageLabel.setText("You choose to exit. Goodbye.");
	}
	
	/**
	 * The main method queries the user for the names of the players, instantiates
	 * a new panel using those names, and does all of the housekeeping to display
	 * the panel.
	 * @param args The command line arguments (not used) 
	 */
	public static void main(String[] args)
	{
		// Create frame and do initialization
		JFrame frame = new JFrame("Connect 4");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
		// Get names of users
		String name1 = JOptionPane.showInputDialog(null, "Enter player 1's name", "Player 1");
		String name2 = JOptionPane.showInputDialog(null, "Enter player 2's name", "Player 2");
	
		// Create new Panel4
		Panel4 panel = new Panel4(name1, name2);
		
		// Add panel, pack and display
		frame.add(panel);
		frame.pack();
		frame.setVisible(true);	
	}
	
	/**
	 * Action listener class for the buttons to add a Piece to a row.
	 * @author Kenneth Marino
	 * @version 1.0 3/14/2012
	 */
	private class ColumnButtonListener implements ActionListener
	{
		// Instance variables
		private int col;
		
		/**
		 * Constructor. Takes in a column. This defines what row the button
		 * will drop onto when it is pressed.
		 * @param col The column the button drops pieces on
		 */
		public ColumnButtonListener(int col)
		{
			this.col = col;
		}
		
		/**
		* Adds a piece to the board and updates the board. Also does the necessary
		* checks for wins and column drops and updates the GUI when changes are 
		* made to board.
		* @param event Event object
		*/
		public void actionPerformed(ActionEvent event)
		{
			// Get current player info
			Player curPlayer;
			if (p1Turn)
				curPlayer = player1;
			else
				curPlayer = player2;
			
			// Make the move
			boolean success = board.move(col, curPlayer.addPiece());
			updateBoard();
			
			// If it failed, print an error message
			if (!success)
			{
				JOptionPane.showMessageDialog(null, "Invalid selection. Please try again");
				p1Turn = !p1Turn;
			}
			else
			{
				Piece[][] pieces = board.getState();
				if (pieces[Board.ROWS-1][col] == null)
					JOptionPane.showMessageDialog(null, "Column " + (col+1) + " dropped!"); 
			}
				
			// Check for a win
			boolean isWin = board.checkForWin();
			if (isWin)
			{
				JOptionPane.showMessageDialog(null, "Congratulations " + curPlayer + ", you won!");
			}
				
			// Switch players
			p1Turn = !p1Turn;
			if (p1Turn)
				PlayerTurn.setText(player1 + "'s turn!");
			else
				PlayerTurn.setText(player2 + "'s turn!");
			
			// Ask to play another game
			if (isWin)
			{
				int again = JOptionPane.showConfirmDialog(null, "Play again?", "Connect 4", JOptionPane.YES_NO_OPTION);
				if (again == JOptionPane.YES_OPTION)
				{
					board.reset();
					updateBoard();
					PlayerTurn.setText(player1 + "'s turn!");
				}
				else
				{
					disableButtons();
				}
			}
		}
	}
}